/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.modules.groovy.support.common;

import org.openide.filesystems.FileObject;
import org.openide.loaders.DataObjectExistsException;
import org.openide.loaders.MultiDataObject;
import org.openide.nodes.CookieSet;
import org.openide.nodes.Node;
import org.openide.util.HelpCtx;
import org.openide.windows.OutputListener;

/** 
 * A superclass of all scripting data objects
 *
 * @author Petr Hamernik
 */
public abstract class ScriptDataObject extends MultiDataObject {
    
    /** Constructor for generic scripting data object.
     */
    public ScriptDataObject(FileObject pf, ScriptLoader loader) throws DataObjectExistsException {
        super(pf, loader);
        init();
    }

    /** Execute the script
     */
    public void runScript() {
        ScriptExecSupport exec = new ScriptExecSupport( this );
        exec.runScript();
    }

    /** Get the script executor. Subclasses must implement this method
     *
     * @return the full path to script interpreter (e.g. "/usr/java/groovy")
     */
    public abstract String getScriptExec();

    /** Try to parse the output line. If there is an error notification
     * at this line, this method should return the OutputListener implementation
     * The easiest way is return instance of {@link ScriptHyperlink}
     *
     * @param line The line to be parsed
     *
     * @return an instance of OutputListener (e.g. ScriptHyperlink) or null if there
     * is no error information at this line
     */
    public abstract OutputListener checkErrorLine( String line );
    
    private void init() {
        CookieSet cookies = getCookieSet();
        cookies.add( new ScriptEditorSupport( this ) );
    }

    @Override
    public HelpCtx getHelpCtx() {
        return HelpCtx.DEFAULT_HELP;
    }
    
    /** Package accessibility for ScriptEditorSupport
     */
    CookieSet getCookieSet0() {
        return getCookieSet();
    }
    
    @Override
    protected Node createNodeDelegate() {
        return new ScriptDataNode(this);
    }
}
