/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.modules.scripting.php.dbginterface.breakpoints;

import javax.swing.Action;
import org.netbeans.spi.viewmodel.NodeActionsProvider;
import org.netbeans.spi.viewmodel.UnknownTypeException;
import org.netbeans.spi.viewmodel.Models;
import org.netbeans.spi.viewmodel.NodeActionsProviderFilter;
import org.openide.filesystems.FileObject;
import org.openide.text.Line;
import org.openide.util.NbBundle;


/**
 * @author Peter Williams
 */
public class BreakpointsActionsProvider implements NodeActionsProviderFilter {

    private static String loc(String key) {
        return NbBundle.getBundle(BreakpointsActionsProvider.class).getString(key);
    }

    private static final Action GO_TO_SOURCE_ACTION = Models.createAction(
        loc("CTL_Breakpoint_GoToSource_Label"), // NOI18N
        new Models.ActionPerformer() {
            public boolean isEnabled(Object node) {
                return true;
            }
            public void perform(Object[] nodes) {
                goToSource((PhpBreakpoint) nodes [0]);
            }
        },
        Models.MULTISELECTION_TYPE_EXACTLY_ONE
    );
//    private static final Action CUSTOMIZE_ACTION = Models.createAction (
//        loc("CTL_Breakpoint_Customize_Label"), // NOI18N
//        new Models.ActionPerformer () {
//            public boolean isEnabled (Object node) {
//                return true;
//            }
//            public void perform (Object[] nodes) {
//                customize ((Breakpoint) nodes [0]);
//            }
//        },
//        Models.MULTISELECTION_TYPE_EXACTLY_ONE
//    );

    public Action[] getActions (NodeActionsProvider original, Object node)
            throws UnknownTypeException {
        Action[] oas = original.getActions(node);
        if(node instanceof PhpBreakpoint) {
            Action[] as = new Action [oas.length + 2];
            as [0] = GO_TO_SOURCE_ACTION;
            as [1] = null;
            System.arraycopy (oas, 0, as, 2, oas.length);
            oas = as;
        }

        return oas;
    }

    public void performDefaultAction(NodeActionsProvider original, Object node) throws UnknownTypeException {
        if(node instanceof PhpBreakpoint) {
            goToSource((PhpBreakpoint) node);
        } else {
            original.performDefaultAction(node);
        }
    }

    private static void goToSource(PhpBreakpoint bp) {
        Line l = bp.getLine();
        l.show(Line.SHOW_GOTO);
    }

//    public static void customize (Breakpoint b) {
//        JComponent c = null;
//        if (b instanceof LineBreakpoint)
//            c = new LineBreakpointPanel ((LineBreakpoint) b);
//        else
//        if (b instanceof FieldBreakpoint)
//            c = new FieldBreakpointPanel ((FieldBreakpoint) b);
//        else
//        if (b instanceof ClassLoadUnloadBreakpoint)
//            c = new ClassBreakpointPanel ((ClassLoadUnloadBreakpoint) b);
//        else
//        if (b instanceof MethodBreakpoint)
//            c = new MethodBreakpointPanel ((MethodBreakpoint) b);
//        else
//        if (b instanceof ThreadBreakpoint)
//            c = new ThreadBreakpointPanel ((ThreadBreakpoint) b);
//        else
//        if (b instanceof ExceptionBreakpoint)
//            c = new ExceptionBreakpointPanel ((ExceptionBreakpoint) b);
//
//        c.getAccessibleContext().setAccessibleDescription(
//                NbBundle.getMessage(BreakpointsActionsProvider.class, "ACSD_Breakpoint_Customizer_Dialog")); // NOI18N
//        HelpCtx helpCtx = HelpCtx.findHelp (c);
//        if (helpCtx == null) {
//            helpCtx = new HelpCtx ("debug.add.breakpoint");  // NOI18N
//        }
//        final Controller[] cPtr = new Controller[] { (Controller) c };
//        final DialogDescriptor[] descriptorPtr = new DialogDescriptor[1];
//        final Dialog[] dialogPtr = new Dialog[1];
//        ActionListener buttonsActionListener = new ActionListener() {
//            public void actionPerformed(ActionEvent ev) {
//                if (descriptorPtr[0].getValue() == DialogDescriptor.OK_OPTION) {
//                    boolean ok = cPtr[0].ok();
//                    if (ok) {
//                        dialogPtr[0].setVisible(false);
//                    }
//                } else {
//                    dialogPtr[0].setVisible(false);
//                }
//            }
//        };
//        DialogDescriptor descriptor = new DialogDescriptor (
//            c,
//            NbBundle.getMessage (
//                BreakpointsActionsProvider.class,
//                "CTL_Breakpoint_Customizer_Title" // NOI18N
//            ),
//            true,
//            DialogDescriptor.OK_CANCEL_OPTION,
//            DialogDescriptor.OK_OPTION,
//            DialogDescriptor.DEFAULT_ALIGN,
//            helpCtx,
//            buttonsActionListener
//        );
//        descriptor.setClosingOptions(new Object[] {});
//        Dialog d = DialogDisplayer.getDefault ().createDialog (descriptor);
//        d.pack ();
//        descriptorPtr[0] = descriptor;
//        dialogPtr[0] = d;
//        d.setVisible (true);
//    }
}
