/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.modules.php.rt.providers.impl.ftp;

import java.net.MalformedURLException;
import java.util.Collection;
import javax.swing.JTextField;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import org.netbeans.modules.php.rt.providers.impl.*;
import org.netbeans.api.project.Project;
import org.netbeans.modules.php.rt.WebServerRegistry;
import org.netbeans.modules.php.rt.spi.providers.Host;
import org.netbeans.modules.php.rt.spi.providers.WebServerProvider;
import org.netbeans.spi.project.support.ant.AntProjectHelper;
import org.netbeans.spi.project.support.ant.EditableProperties;


/**
 * @author  ads
 */
public class FtpProjectCustomizer extends AbstractProjectCustomizerComponent {

    private static final long serialVersionUID = 6233510568797205953L;

    public FtpProjectCustomizer(WebServerProvider provider, Project project) {
        myProvider = provider;
        myProject = project;
        initComponents();
        init();
    }

    protected void init() {
        DocumentListener listener = new ContextListener();
        getContextPath().getDocument().addDocumentListener(listener);
    }

    public void store(EditableProperties properties) {
        String context = getContextPath().getText();
        properties.setProperty(AbstractProjectConfigProvider.CONTEXT, context);
    }

    public void read(EditableProperties properties) {
        myProperties = properties;
        String context = getProperties().getProperty(AbstractProjectConfigProvider.CONTEXT);
        myContextPath.setText(context);
        updateHost();
        contextUpdated();
    }

    protected void contextUpdated() {

        Host hostCandidate = getHost();
        if (hostCandidate != null && hostCandidate instanceof FtpHostImpl) {
            FtpHostImpl host = (FtpHostImpl) hostCandidate;

            updateHttpPath(host);

            updateFtpPath(host);
        }
        if (isContentValid()) {
            fireCustomizerPropertyChanged();
        }
    }

    private boolean isContentValid() {
        // use fireCustomizerErrorMessage(""); here in case of any error
        return true;
    }

    private void updateHttpPath(FtpHostImpl host) {
        try {
            String httpUrl = createPreviewUrl(FtpHostImpl.Helper.getHttpUrl(host));
            getHttpPath().setText(httpUrl);
            getHttpPath().setToolTipText(httpUrl);
        } catch (MalformedURLException ex) {
            getHttpPath().setText(ex.toString());
        }
    }

    private void updateFtpPath(FtpHostImpl host) {
        try {
            
            String ftpUrl = createPreviewUrl(FtpHostImpl.Helper.getFtpUrl(host));
            
            getFtpPath().setText(ftpUrl.toString());
            getFtpPath().setToolTipText(ftpUrl);
        } catch (MalformedURLException ex) {
            getFtpPath().setText(ex.toString());
        }
    }

    private String createPreviewUrl(String baseUrl) throws MalformedURLException {
        String context = getContextPath().getText();
        return FtpHostImpl.Helper.addSubdirectoryToUrl(baseUrl, context);
    }

    /**
     * reloads host from WebServerRegistry if host id in properties was changed
     */
    private Host updateHost() {
        String hostId = getProperties().getProperty(WebServerProvider.HOST_ID);
        if (myHost != null && myHost.getId().equals(hostId)) {
            return myHost;
        }
        Collection<Host> collection = WebServerRegistry.getInstance().getHosts();
        for (Host host : collection) {
            if (host.getId().equals(hostId)) {
                myHost = host;
                break;
            }
        }
        return myHost;
    }

    private class ContextListener implements DocumentListener {

        /* (non-Javadoc)
         * @see javax.swing.event.DocumentListener#changedUpdate(javax.swing.event.DocumentEvent)
         */
        public void changedUpdate(DocumentEvent arg0) {
            contextUpdated();
        }

        /* (non-Javadoc)
         * @see javax.swing.event.DocumentListener#insertUpdate(javax.swing.event.DocumentEvent)
         */
        public void insertUpdate(DocumentEvent arg0) {
            contextUpdated();
        }

        /* (non-Javadoc)
         * @see javax.swing.event.DocumentListener#removeUpdate(javax.swing.event.DocumentEvent)
         */
        public void removeUpdate(DocumentEvent arg0) {
            contextUpdated();
        }
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:initComponents
    private void initComponents() {

        myContextPathLabel = new javax.swing.JLabel();
        myContextPath = new javax.swing.JTextField();
        myHttpPath = new javax.swing.JTextField();
        myFtpPath = new javax.swing.JTextField();
        myHttpPathLbl = new javax.swing.JLabel();
        myFtpPathLbl = new javax.swing.JLabel();

        myContextPathLabel.setLabelFor(myContextPath);
        java.util.ResourceBundle bundle = java.util.ResourceBundle.getBundle("org/netbeans/modules/php/rt/providers/impl/ftp/Bundle"); // NOI18N
        org.openide.awt.Mnemonics.setLocalizedText(myContextPathLabel, bundle.getString("LBL_Context_Path")); // NOI18N

        myContextPath.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                myContextPathActionPerformed(evt);
            }
        });

        myHttpPath.setEditable(false);

        myFtpPath.setEditable(false);

        myHttpPathLbl.setText(org.openide.util.NbBundle.getMessage(FtpProjectCustomizer.class, "LBL_Http_Path")); // NOI18N

        myFtpPathLbl.setText(org.openide.util.NbBundle.getMessage(FtpProjectCustomizer.class, "LBL_Ftp_Path")); // NOI18N

        org.jdesktop.layout.GroupLayout layout = new org.jdesktop.layout.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(myContextPathLabel)
                    .add(myHttpPathLbl)
                    .add(myFtpPathLbl))
                .add(26, 26, 26)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING)
                    .add(myHttpPath, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 279, Short.MAX_VALUE)
                    .add(org.jdesktop.layout.GroupLayout.LEADING, myFtpPath, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 279, Short.MAX_VALUE)
                    .add(org.jdesktop.layout.GroupLayout.LEADING, myContextPath, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 279, Short.MAX_VALUE))
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .addContainerGap()
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(myContextPathLabel)
                    .add(myContextPath, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(myHttpPathLbl)
                    .add(myHttpPath, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .add(8, 8, 8)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(myFtpPathLbl)
                    .add(myFtpPath, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addContainerGap(org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        myContextPathLabel.getAccessibleContext().setAccessibleName(bundle.getString("A11_Context_PathLbl")); // NOI18N
        myContextPath.getAccessibleContext().setAccessibleDescription(bundle.getString("A11_Context_Path")); // NOI18N
        myHttpPath.getAccessibleContext().setAccessibleDescription(org.openide.util.NbBundle.getMessage(FtpProjectCustomizer.class, "A11_Http_Path")); // NOI18N
        myFtpPath.getAccessibleContext().setAccessibleDescription(org.openide.util.NbBundle.getMessage(FtpProjectCustomizer.class, "A11_Ftp_Path")); // NOI18N
        myHttpPathLbl.getAccessibleContext().setAccessibleName(org.openide.util.NbBundle.getMessage(FtpProjectCustomizer.class, "A11_Http_PathLbl")); // NOI18N
        myFtpPathLbl.getAccessibleContext().setAccessibleName(org.openide.util.NbBundle.getMessage(FtpProjectCustomizer.class, "A11_Ftp_PathLbl")); // NOI18N
    }// </editor-fold>//GEN-END:initComponents

    private void myContextPathActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_myContextPathActionPerformed
            }//GEN-LAST:event_myContextPathActionPerformed


    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JTextField myContextPath;
    private javax.swing.JLabel myContextPathLabel;
    private javax.swing.JTextField myFtpPath;
    private javax.swing.JLabel myFtpPathLbl;
    private javax.swing.JTextField myHttpPath;
    private javax.swing.JLabel myHttpPathLbl;
    // End of variables declaration//GEN-END:variables

    // End of variables declaration
    // End of variables declaration
    // End of variables declaration
    // End of variables declaration
    protected JTextField getContextPath() {
        return myContextPath;
    }

    protected JTextField getHttpPath() {
        return myHttpPath;
    }

    protected JTextField getFtpPath() {
        return myFtpPath;
    }

    private WebServerProvider getProvider() {
        return myProvider;
    }

    private Project getProject() {
        return myProject;
    }

    private Host getHost() {
        return myHost;
    }

    private EditableProperties getProperties() {
        return myProperties;
    }

    private AntProjectHelper getAntProjectHelper(Project project) {
        return project.getLookup().lookup(AntProjectHelper.class);
    }

    private Project myProject;

    private EditableProperties myProperties;

    private Host myHost;

    private WebServerProvider myProvider;
}
