/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.modules.xslt.model.impl;

import java.util.List;

import org.netbeans.modules.xslt.model.AttributeSet;
import org.netbeans.modules.xslt.model.Copy;
import org.netbeans.modules.xslt.model.SequenceElement;
import org.netbeans.modules.xslt.model.XslComponent;
import org.netbeans.modules.xslt.model.XslReference;
import org.netbeans.modules.xslt.model.XslVisitor;
import org.netbeans.modules.xslt.model.enums.TBoolean;
import org.w3c.dom.Element;


/**
 * @author ads
 *
 */
class CopyImpl extends ValidationCopyNsSpecImpl implements Copy {

    CopyImpl( XslModelImpl model, Element element ) {
        super( model , element );
    }
    
    CopyImpl( XslModelImpl model ){
        super( model , XslElements.COPY );
    }

    /* (non-Javadoc)
     * @see org.netbeans.modules.xslt.model.impl.XslComponentImpl#accept(org.netbeans.modules.xslt.model.XslVisitor)
     */
    @Override
    public void accept( XslVisitor visitor )
    {
        visitor.visit( this );
    }

    /* (non-Javadoc)
     * @see org.netbeans.modules.xslt.model.impl.XslComponentImpl#getComponentType()
     */
    @Override
    public Class<? extends XslComponent> getComponentType()
    {
        return Copy.class;
    }

    /* (non-Javadoc)
     * @see org.netbeans.modules.xslt.model.UseAttributesSetsSpec#getUseAttributeSets()
     */
    public List<XslReference<AttributeSet>> getUseAttributeSets() {
        return resolveGlobalReferenceList( AttributeSet.class, 
                XslAttributes.USE_ATTRIBUTE_SETS );
    }

    /* (non-Javadoc)
     * @see org.netbeans.modules.xslt.model.UseAttributesSetsSpec#setUseAttributeSets(java.util.Collection)
     */
    public void setUseAttributeSets( List<XslReference<AttributeSet>> collection ) {
        setAttributeList( XslAttributes.USE_ATTRIBUTE_SETS , collection );
    }
    
    /* (non-Javadoc)
     * @see org.netbeans.modules.xslt.model.SequenceConstructor#addSequenceChild(org.netbeans.modules.xslt.model.SequenceElement, int)
     */
    public void addSequenceChild( SequenceElement element, int position ) {
        insertAtIndex( SEQUENCE_ELEMENT, element , position );
    }

    /* (non-Javadoc)
     * @see org.netbeans.modules.xslt.model.SequenceConstructor#appendSequenceChild(org.netbeans.modules.xslt.model.SequenceElement)
     */
    public void appendSequenceChild( SequenceElement element ) {
        appendChild( SEQUENCE_ELEMENT, element );
    }

    /* (non-Javadoc)
     * @see org.netbeans.modules.xslt.model.SequenceConstructor#getSequenceChildren()
     */
    public List<SequenceElement> getSequenceChildren() {
        return getChildren( SequenceElement.class );
    }

    /* (non-Javadoc)
     * @see org.netbeans.modules.xslt.model.SequenceConstructor#removeSequenceChild(org.netbeans.modules.xslt.model.SequenceElement)
     */
    public void removeSequenceChild( SequenceElement element ) {
        removeChild( SEQUENCE_ELEMENT, element );
    }
    
    /* (non-Javadoc)
     * @see org.netbeans.modules.xslt.model.InheritNamespacesSpec#getInheritNamespaces()
     */
    public TBoolean getInheritNamespaces() {
        return TBoolean.forString( getAttribute( 
                XslAttributes.INHERIT_NAMESPACES ));
    }

    /* (non-Javadoc)
     * @see org.netbeans.modules.xslt.model.InheritNamespacesSpec#setInheritNamespaces(org.netbeans.modules.xslt.model.enums.TBoolean)
     */
    public void setInheritNamespaces( TBoolean value ) {
        setAttribute( XslAttributes.INHERIT_NAMESPACES, value );
    }

}
