/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.modules.xslt.model.impl;

import java.util.List;

import javax.xml.namespace.QName;

import org.netbeans.modules.xslt.model.Param;
import org.netbeans.modules.xslt.model.Template;
import org.netbeans.modules.xslt.model.XslComponent;
import org.netbeans.modules.xslt.model.XslVisitor;
import org.w3c.dom.Element;


/**
 * @author ads
 *
 */
class TemplateImpl extends MatchableQNameableSeqCtorImpl implements Template {

    TemplateImpl( XslModelImpl model, Element element ) {
        super( model , element );
    }

    TemplateImpl( XslModelImpl model ) {
        super( model , XslElements.TEMPLATE );
    }

    /* (non-Javadoc)
     * @see org.netbeans.modules.xslt.model.impl.XslComponentImpl#accept(org.netbeans.modules.xslt.model.XslVisitor)
     */
    @Override
    public void accept( XslVisitor visitor )
    {
        visitor.visit( this );
    }

    /* (non-Javadoc)
     * @see org.netbeans.modules.xslt.model.impl.XslComponentImpl#getComponentType()
     */
    @Override
    public Class<? extends XslComponent> getComponentType()
    {
        return Template.class;
    }

    /* (non-Javadoc)
     * @see org.netbeans.modules.xslt.model.Template#getPriority()
     */
    public Double getPriority() {
        String value = getAttribute( XslAttributes.PRIORITY );
        Double numValue = Double.parseDouble( value );
        return numValue;
    }

    /* (non-Javadoc)
     * @see org.netbeans.modules.xslt.model.Template#setPriority(java.lang.Double)
     */
    public void setPriority( Double priority ) {
        setAttribute( XslAttributes.PRIORITY, priority );
    }

    /* (non-Javadoc)
     * @see org.netbeans.modules.xslt.model.ParamContainer#addParam(org.netbeans.modules.xslt.model.Param, int)
     */
    public void addParam( Param param, int position ) {
        insertAtIndex( PARAM_PROPERTY, param, position );
    }

    /* (non-Javadoc)
     * @see org.netbeans.modules.xslt.model.ParamContainer#appendParam(org.netbeans.modules.xslt.model.Param)
     */
    public void appendParam( Param param ) {
        addBefore( PARAM_PROPERTY , param , SEQUENCE_ELEMENTS );
    }

    /* (non-Javadoc)
     * @see org.netbeans.modules.xslt.model.ParamContainer#getParams()
     */
    public List<Param> getParams() {
        return getChildren( Param.class );
    }

    /* (non-Javadoc)
     * @see org.netbeans.modules.xslt.model.ParamContainer#removeParam(org.netbeans.modules.xslt.model.Param)
     */
    public void removeParam( Param param ) {
        removeChild( PARAM_PROPERTY , param );
    }

    /* (non-Javadoc)
     * @see org.netbeans.modules.xslt.model.Template#getMode()
     */
    public List<QName> getMode() {
        return getQNameList( getAttribute( XslAttributes.MODES ) );
    }

    /* (non-Javadoc)
     * @see org.netbeans.modules.xslt.model.Template#setMode(java.util.List)
     */
    public void setMode( List<QName> mode ) {
        setAttribute( XslAttributes.MODES, mode );
    }
    
    /* (non-Javadoc)
     * @see org.netbeans.modules.xslt.model.AsSpec#getAs()
     */
    public String getAs() {
        return getAttribute( XslAttributes.AS );
    }

    /* (non-Javadoc)
     * @see org.netbeans.modules.xslt.model.AsSpec#setAs(java.lang.String)
     */
    public void setAs( String value ) {
        setAttribute( XslAttributes.AS, value );
    }

}
